@@ -1,5 +1,5 @@
 '''
-Copyright (C) 2024 Mo Zhou <lumin@debian.org>
+Copyright (C) 2024-2025 Mo Zhou <lumin@debian.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
@@ -144,7 +144,13 @@ class AbstractFrontend():
         _check(self.session)
 
     def __call__(self, *args, **kwargs):
-        return self.query(*args, **kwargs)
+        try:
+            res = self.query(*args, **kwargs)
+            return res
+        except Exception as e:
+            # this will only appear in dumped session files
+            self.update_session({'role': 'system', 'content': str(e)})
+            raise e
 
     def dump(self):
         fpath = os.path.join(self.debgpt_home, str(self.uuid) + '.json')
@@ -210,9 +216,13 @@ class OpenAIFrontend(AbstractFrontend):
             exit(1)
         self.client = OpenAI(api_key=args.openai_api_key,
                              base_url=args.openai_base_url)
-        self.session.append({"role": "system", "content": self.system_message})
         self.model = args.openai_model
-        self.kwargs = {'temperature': args.temperature, 'top_p': args.top_p}
+        # XXX: some models do not support system messages yet. nor temperature.
+        if self.model not in ('o1-mini', 'o1-preview'):
+            self.session.append({"role": "system", "content": self.system_message})
+            self.kwargs = {'temperature': args.temperature, 'top_p': args.top_p}
+        else:
+            self.kwargs = {}
         if args.verbose:
             console.log(f'{self.NAME}> model={repr(self.model)}, ' +
                         f'temperature={args.temperature}, top_p={args.top_p}.')
@@ -554,6 +564,29 @@ class ZMQFrontend(AbstractFrontend):
         return self.session[-1]['content']
 
 
+def get_username():
+    try:
+        import getpass
+        return getpass.getuser()
+    except:
+        pass
+    try:
+        import pwd
+        return pwd.getpwuid(os.getuid())[0]
+    except:
+        pass
+    try:
+        return os.getlogin()
+    except:
+        pass
+    # common shell env
+    for env in ('USER', 'USERNAME', 'LOGNAME'):
+        if u := os.environ.get(env):
+            return u
+    # final fallback
+    return 'user'
+
+
 def create_frontend(args):
     if args.frontend == 'zmq':
         frontend = ZMQFrontend(args)
@@ -639,9 +672,10 @@ def interact_with(f: AbstractFrontend) -> None:
         )
 
     # loop
+    user = get_username()
     try:
         while text := prompt_session.prompt(
-                f'{os.getlogin()}[{max(1, len(f.session))}]> '):
+                f'{user}[{max(1, len(f.session))}]> '):
             # parse escaped interaction commands
             if text.startswith('/'):
                 cmd = shlex.split(text)
